import zlib
import numpy as np
from numba import njit


def mask_to_proto(arr, proto_obj, compress=False):
    proto_obj.shape.extend(arr.shape)
    data = np.require(arr, dtype=np.int8).tobytes()
    if compress:
        data = zlib.compress(data)
    proto_obj.data = data
    proto_obj.zlib = compress

def proto_to_mask(proto_obj):
    data = proto_obj.data
    if proto_obj.zlib:
        data = zlib.decompress(data)
    return np.frombuffer(data, dtype=np.int8).reshape(proto_obj.shape)


@njit
def detection_double_thresh(feature_id, inner_thresh_mask, outer_thresh_mask, entire_globe):

    # Find datapoint over upper thresh that trigger feature detection
    features = np.zeros_like(inner_thresh_mask, dtype = np.int64)
    # Classical threshold algorithm for lower thresh
    # Looping through all gridpoints in space
    for i in range(len(outer_thresh_mask[:,0,0])):
        for j in range(len(outer_thresh_mask[0,:,0])):
            for k in range(len(outer_thresh_mask[0,0,:])):
                # Check local_selection_criterion
                if outer_thresh_mask[i, j, k] :
                    #if t==0 : print(i,j,k)
                    # Check already visited neighbours for feature and merge to one feature, else assign new feature
                    neighbour_candidates = []
                    for (x, y, z) in [(i+di, j+dj, k+dk) for di in (-1, 0) for dj in (-1, 0) for dk in (-1, 0) if not (di == dj == dk == 0)]:
                        # Excluding array neighbours with index -1 for lat and lev, for lon only if not entire globe:
                        if (not entire_globe and not (x == -1 or y == -1 or z == -1)) or (entire_globe and not (x == -1 or y == -1)):
                            
                            # Check if neighbour is feature and fullfills the local_homogenity_criterion
                            if (features[x, y, z] != 0):# and (np.sign(data[i, j, k]) == np.sign(data[x, y, z])):# and local_homogenity_criterion(data[i, j, k], data[x, y, z]):
                                # Keeping track of visited neighbours
                                neighbour_candidates.append(features[x, y, z])

                    neighbour_candidates = list(set(neighbour_candidates))
                    # Dealing with potential neighbouring features:
                    # 1. Check if there are no neighbours
                    if len(neighbour_candidates) == 0:
                        # Set new feature for gridpoint without neighbours
                        features[i, j, k] = feature_id
                        feature_id += 1
                        
                        # 2. Check if there is one neighbours
                    elif len(neighbour_candidates) == 1:
                        # Add coord to neighbour candidate
                        # if neighbour_candidates[0] in features:
                        features[i, j, k] = neighbour_candidates[0]

                        # 3. Check if there is more than one neighbour
                    else:
                            # Set new feature and then check neighbours for further action
                        min_id = min(neighbour_candidates)
                        for n in neighbour_candidates:
                            # Merge neighbours into new feature
                            for (a,b,c) in np.nditer(np.where(features == n)):
                                features[a,b,c] = min_id
                        features[i, j, k] = min_id
        
    # Find Features to delete, that do not have gridpoint crossing the upper thresh
    upper_and_lower_thresh_features = features * inner_thresh_mask
    keep_features = np.unique(upper_and_lower_thresh_features)
    all_features = np.unique(features)
    del_features = set(all_features) - set(keep_features)

    # Delte all del_features from feature array
    for id in del_features:
        for (a,b,c) in np.nditer(np.where(features == id)):
            features[a,b,c] = 0
    
    return keep_features, features, upper_and_lower_thresh_features

